@@ -33,6 +33,8 @@ import os
 import pytest
 import requests
 
+EXECUTION_ERROR = "Execution error"
+
 program_file = str("{program_to_be_tested}").replace(".py", "")
 
 program = importlib.import_module(program_file)
@@ -49,45 +51,66 @@ with open("test_values.txt", "r", encoding="utf-8") as file:
         output = []
         partial_output = ""
 
-        def mock_input(s):
-            output.append(s)
+        def mock_input(*args):
+            if len(args) > 1:
+                raise ValueError(EXECUTION_ERROR)
+            elif len(args) == 1:
+                output.append(args[0])
             return input_values.pop(0)
 
         def generate_output(*args, **kwargs):
-          nonlocal partial_output
-          # TODO what happens if \\n is included in end?
+            nonlocal partial_output
+            # TODO what happens if \\n is included in end?
 
-          # if there is an end keyowrd argument, output is stored
-          # in a string instead of added to the output
-          if "end" in args[1]:
-            if len(args[0]) != 0:
-              partial_output += str(args[0][0])
-              for j in range(1, len(args[0])):
-                partial_output = partial_output + " " + str(args[0][j])
-            partial_output += str(args[1]["end"])
-          else:
-            string = partial_output
-            # if there are several arguments in print(), they are added to the ouput
-            if len(args[0]) == 0:
-              string += ""
+            # if there is an end keyowrd argument, output is stored
+            # in a string instead of added to the output
+            if "end" in args[1]:
+                if len(args[0]) != 0:
+                    partial_output += str(args[0][0])
+                    for j in range(1, len(args[0])):
+                        partial_output = partial_output + " " + str(args[0][j])
+                partial_output += str(args[1]["end"])
             else:
-              string += str(args[0][0])
-              for j in range(1, len(args[0])):
-                string += " " + str(args[0][j])
-            partial_output = ""
-            output.append(string)
+                string = partial_output
+                # if there are several arguments in print(), they are added to the ouput
+                if len(args[0]) == 0:
+                    string += ""
+                else:
+                    string += str(args[0][0])
+                    for j in range(1, len(args[0])):
+                        string += " " + str(args[0][j])
+                partial_output = ""
+                output.append(string)
 
         program.input = mock_input
 
+        program.print = lambda *args, **kwargs: generate_output(args, kwargs)
+
         try:
             program.random.randrange = lambda *args : input_values.pop(0)
         except:
             pass
 
-        program.print = lambda *args, **kwargs: generate_output(args, kwargs)
+        try:
+            program.randrange = lambda *args : input_values.pop(0)
+        except:
+            pass
+
+        try:
+            program.random.choice = lambda *args : input_values.pop(0)
+        except:
+            pass
+
+        try:
+            program.choice = lambda *args : input_values.pop(0)
+        except:
+            pass
 
         program.main()
 
+        if partial_output != "":
+            output.append(partial_output)
+
         with open("obtained_result.txt", "w", encoding="utf-8") as file:
             # saved as json because it is a list
             json.dump(output, file, ensure_ascii=False)
@@ -98,6 +121,8 @@ with open("test_values.txt", "r", encoding="utf-8") as file:
 
 
 def main():
+    EXECUTION_ERROR = "Execution error"
+
     parser = argparse.ArgumentParser(
         description="Testing tool for some of the programming exercises in mclibre.org's Python course available at http://www.mclibre.org/consultar/python/"
     )
@@ -164,7 +189,9 @@ def main():
         print()
         print("Please, wait until all tests have been executed.")
         print()
-        print("Now, PyTest will print some messages while the tests are being executed.")
+        print(
+            "Now, PyTest will print some messages while the tests are being executed."
+        )
         print()
         print("A final report will be shown after.")
         print()
@@ -187,11 +214,14 @@ def main():
             root = tree.getroot()
             for neighbor in root.iter("testsuite"):
                 if int(neighbor.attrib["failures"]) > 0:
-                    with open("obtained_result.txt", "r", encoding="utf-8") as file:
-                        # file is saved as json and is read as a list
-                        texto = json.load(file)
-                        print(texto)
-                    errorReport += [[i["input"], i["output"], texto]]
+                    try:
+                        with open("obtained_result.txt", "r", encoding="utf-8") as file:
+                            # file is saved as json and is read as a list
+                            texto = json.load(file)
+                            # print(texto)
+                        errorReport += [[i["input"], i["output"], texto, ""]]
+                    except:
+                        errorReport += [[i["input"], i["output"], "", EXECUTION_ERROR]]
             if os.path.isfile("obtained_result.txt"):
                 os.remove("obtained_result.txt")
 
@@ -209,6 +239,9 @@ def main():
         print("-+-+-+-+-+-+-+-+- MCLIBRE PYTHON TESTING -+-+-+-+-+-+-+-+-")
         print("-+-+-+-+-+-+-+-+-         RESULTS        -+-+-+-+-+-+-+-+-")
         print()
+        print(f"Tested program: {args.to_be_tested_py}")
+        print(f"MPTC number:    {args.exercise_id}")
+        print()
         if len(values["result"]) > 1:
             print(f"{len(values['result'])} tests have been executed.")
         else:
@@ -239,23 +272,30 @@ def main():
                     print()
                 else:
                     print("  Tested values:   None")
-                if len(i[1]) != len(i[2]):
-                    print("  The program produces an incorrect number of outputs.")
-                for j in range(min(len(i[1]), len(i[2]))):
-                    if i[1][j] != i[2][j]:
+                if i[3] == EXECUTION_ERROR:
+                    print(
+                        "  Your program could not be executed properly. Please, check manually."
+                    )
+                else:
+                    if len(i[1]) != len(i[2]):
+                        print("  The program produces an incorrect number of outputs.")
+                    for j in range(min(len(i[1]), len(i[2]))):
+                        if i[1][j] != i[2][j]:
+                            print()
+                            print(f'  Expected result: "{i[1][j]}"')
+                            print(f'  Obtained result: "{i[2][j]}"')
+                    for j in range(
+                        min(len(i[1]), len(i[2])), max(len(i[1]), len(i[2]))
+                    ):
                         print()
-                        print(f'  Expected result: "{i[1][j]}"')
-                        print(f'  Obtained result: "{i[2][j]}"')
-                for j in range(min(len(i[1]), len(i[2])), max(len(i[1]), len(i[2]))):
-                    print()
-                    if 0 <= j < len(i[1]):
-                        print(f'  Expected result: "{i[1][j]}"')
-                    else:
-                        print(f"  No result was expected.")
-                    if 0 <= j < len(i[2]):
-                        print(f'  Obtained result: "{i[2][j]}"')
-                    else:
-                        print(f"  No result was obtained.")
+                        if 0 <= j < len(i[1]):
+                            print(f'  Expected result: "{i[1][j]}"')
+                        else:
+                            print(f"  No result was expected.")
+                        if 0 <= j < len(i[2]):
+                            print(f'  Obtained result: "{i[2][j]}"')
+                        else:
+                            print(f"  No result was obtained.")
 
 
 if __name__ == "__main__":
